#pragma once

#define SBUS_CHANNEL	16
#define SBUS_COUNT		25

typedef struct {
	unsigned short t1;
	unsigned short t2;
	unsigned short t3;
	unsigned short t4;
	unsigned short t5;
	unsigned short t6;
	unsigned short t7;
	unsigned short t8;
	unsigned short switch_sa;
	unsigned short left_knob;
	unsigned short right_knob;
	unsigned short switch_sb;
	unsigned short switch_sd;
	unsigned short switch_se;
	unsigned short buttons;
}T20MonitorData;

typedef struct {
	unsigned short sbus1[16];
	unsigned short sbus2[16];
}DoubleSBUSData;

typedef struct {
	unsigned short x_axis;
	unsigned short y_axis;
	unsigned short z_axis;
	unsigned short x_rotation;
	unsigned short y_rotation;
	unsigned short z_rotation;
	unsigned short slider;
	unsigned short dial;
	unsigned short hat;
	unsigned short buttons[2];
}HID_GAME_CONTROLLER_STRUCTURE;

typedef struct {
	unsigned short t1;
	unsigned short t2;
	unsigned short t3;
	unsigned short t4;
	unsigned short t5;
	unsigned short t6;
	unsigned short t7;
	unsigned short t8;
	unsigned short switch_sa;
	unsigned short left_knob;
	unsigned short right_knob;
	unsigned short switch_sb;
	unsigned short switch_sd;
	unsigned short switch_se;
	unsigned short buttons;
}HID_COMMUNICATION_STRUCTURE;

/*
	* @brief	This function is used to get an instruction that requests the port to upload data
	* @param	cmd: pointer to the memory address of the instruction
	*			type: the type of requested data.
	*				  0: physical channel data
	*				  1: double sbus data
	*				  2: physical channel data and double sbus data
	*			ms: frequency of data upload,range is [10,200]
	* @retval	None
	*/
void request_upload_port_data(unsigned char* cmd, unsigned char type, unsigned char ms);
/*
* @brief	This function is used to get an instruction that stops the port to upload data
* @param	cmd: pointer to the memory address of the instruction
* @retval	None
*/
void stop_upload_port_data(unsigned char* cmd);
/*
* @brief	This function is used to parse port data
* @param	data: port data
*			dataLen: the length of port data.
*			monitor_data: pointer to the memory of structure that stores physical channel data
*			dsbus_data: pointer to the memory of structure that stores double sbus data
* @retval	return 1 if parsing is correct,or return 0 if error
*/
int parse_port_data(unsigned char* data, unsigned char dataLen, T20MonitorData* monitor_data, DoubleSBUSData* dsbus_data);
/*
* @brief	This function is used to parse data of HID game-controller device
* @note		You must set the hid mode to HID game-controller mode through ground station configuration software
* @param	hid_game_controller_data: pointer to the memory address of the structure that stores data of HID game-controller device
*			data: data of HID game-controller device
*			dataLen: the length of data
* @retval	return 1 if parsing is correct,or return 0 if error
*/
int parse_hid_game_controller_device(HID_GAME_CONTROLLER_STRUCTURE* hid_game_controller_data, unsigned char* data, unsigned char dataLen);
/*
* @brief	This function is used to parse data of HID compatible device
* @note		You must set the hid mode to HID transmisson mode through ground station configuration software
* @param	hid_compatible_data: pointer to the memory address of the structure that stores data of HID compatible device
*			data: data of HID compatible device
*			dataLen: the length of data
* @retval	return 1 if parsing is correct,or return 0 if error
*/
int parse_hid_compatible_device(HID_COMMUNICATION_STRUCTURE* hid_compatible_data, unsigned char* data, unsigned char dataLen);
/*
* @brief	This function is used to convert sbus data to standard sbus data which range is [0,2047]
* @param	destination: pointer to the memory address of destination
*			source: pointer to the memory address of source data
* @retval	None
*/
void convert_std_sbus_data(unsigned short* destination, unsigned short* source);
/*
* @brief	This function is used to encode standard sbus data as transmission data
* @param	cmd: pointer to the memory address of the instruction
*			type: the type of requested data.
*				  0: physical channel data
*				  1: double sbus data
*				  2: physical channel data and double sbus data
*			ms: frequency of data upload,range is [10,200]
* @retval	None
*/
void encode_sbus_data_v1(unsigned char* sbus, unsigned short* std_data);
/*
* @brief	This function is used to get an instruction that requests the port to upload data
* @param	cmd: pointer to the memory address of the instruction
*			type: the type of requested data.
*				  0: physical channel data
*				  1: double sbus data
*				  2: physical channel data and double sbus data
*			ms: frequency of data upload,range is [10,200]
* @retval	None
*/
void encode_sbus_data_v2(unsigned char* sbus, unsigned short* std_data);

